package model;

import java.util.StringTokenizer;

public class Date
implements Comparable<Date>, Operable
{
	private int day;
    private int month;
    private int year;
    public Date() {
        day = 0;
        month = 0;
        year = 0;
    }
    /**
     * initialise une date
     * 
     * @param d un entier qui fixe la valeur du jour dans le mois
     * @param m un entier qui fixe la valeur du mois dans l'année
     * @param y un entier qui fixe la valeur de l'année
     */
    public Date(int d, int m, int y) {
        day = d;
        month = m;
        year = y;
    }
	public Date(String s) throws Exception {
		if (!isCorrect(s)) throw new Exception("Date incorrecte : " + s);
		StringTokenizer st = new StringTokenizer(s, "/");
		day   = Integer.parseInt(st.nextToken());
		month = Integer.parseInt(st.nextToken());
		year  = Integer.parseInt(st.nextToken());
	}
    public int day() {
		return day;
	}
	public int month() {
		return month;
	}
	public int year() {
		return year;
	}
	public void day(int d) {
		day = d;
	}
	public void month(int m) {
		month = m;
	}
	public void year(int y) {
		year = y;
	}
    /**
     * construit un texte notant la date
     * 
     * @return une chaîne de caractères au format jj/mm/aaaa
     */
    public String toString() {
        String s = "";
        s = s + day;
        s += '/';
        if (month < 10) s += '0';
        s += month;
        s += '/';
        s += year;
        return s;
    }
    /**
     * change la date en son lendemain
     */
    public void oneDayLater() {
        if (day < numberOfDays(month, year)) {
            day ++;
        } else {
            day = 1;
            if (month < 12) {
                month ++;
            } else {
                month = 1;
                year ++;
            }
        }
    }
    /**
     * change la date en sa veille
     */
    public void oneDayEarlier() {
        if (day > 1) {
            day --;
        } else {  
            if (month >1) {
                month --;
            } else {
                month = 12;
                year --;
            }
            day = numberOfDays(month, year);
        }
    }
    /**
     * détermine le nombre de jours d'un mois de l'année
     * 
     * @param m un entier indiquant le mois auquel on s'intéresse
     * @param y un entier indiquant l'année à laquelle on s'intéresse
     * 
     * @return un entier indiquant la longueur du mois m de l'année a
     */
    public static int numberOfDays(int m, int y) {
        switch(m) {
            case 1  :
            case 3  :
            case 5  :
            case 7  :
            case 8  :
            case 10 :
            case 12 :
                return 31;
            case 4 :
            case 6 :
            case 9 :
            case 11 :
                return 30;
            case 2 :
              //if (leapYear(a) return 29; else return 28;
                return (leapYear(y)) ? 29 : 28;
            default :
                return 0;
        }
    }
    /**
     * indique si une année est bissextile
     * (c'est-à-dire si elle compte 366 jours au lieu de 365)
     * 
     * @param a un entier qui précise l'année à laquelle on s'intéresse
     * 
     * @return un booléen vrai quand l'année a est bissextile, faux dans le cas contraire
     */
    public static boolean leapYear(int year) {
        if (year % 400 == 0) return true;
        if (year % 4 == 0 && year % 100 != 0) return true;
        return false;
    }
	public static boolean isCorrect(String s) {
		if (s.matches("^[0-3]{0,1}[0-9]/[0-1][0-9]/[0-9]{4}$")) {
			StringTokenizer st = new StringTokenizer(s, "/");
			int d, m, y;
			d = Integer.parseInt(st.nextToken());
			if (d < 0 || d > 31) return false;
			m = Integer.parseInt(st.nextToken());
			if (m < 0 || m > 12) return false;
			y = Integer.parseInt(st.nextToken());
			if (y < 1900 || y > 2050) return false;
			if (d > numberOfDays(m, y)) return false;
			return true;
		} else {
			return false;
		}
	}
	public void performs(Operation op) throws Exception {
		op.doItOn(this, 0);
	}
	public void performs(Operation op, int margin) throws Exception {
		op.doItOn(this, margin);
	}
	public int compareTo(Date o) {
		if (year == o.year) {
			if (month == o.month) {
				return day - o.day;
			} else {
				return month - o.month;
			}
		}
		return year - o.year;
	}
}
