package model;

import java.io.File;
import java.util.Iterator;
import java.util.LinkedList;

public class ToDoList
extends ToDo
implements Iterable<ToDo> {
	public static final char CODE = 'L';
	private LinkedList<ToDo> content;
	private File file;
	private boolean changed;
	public ToDoList() {
		super();
		content = new LinkedList<ToDo>();
		changed = false;
		file = null;
	}
	public ToDoList(String name) {
		super(name);
		content = new LinkedList<ToDo>();
		changed = false;
		file = null;
	}
	public boolean hasChanged() {
		return changed;
	}
	public void setChanged() {
		changed = true;
	}
	public boolean saveAs(File f) {
		file = f;
		String fileName = f.getAbsolutePath();
		int dotPos = fileName.lastIndexOf('.');
		String extension = fileName.substring(dotPos + 1);
		FileType ft = FileType.typeForExtension(extension);
		Saver recorder = new Saver(this);
		boolean saved = recorder.saveAs(fileName, ft);
		if (saved) changed = false;
		return saved;
	}
	public boolean save() {
		return saveAs(file);
	}
	public boolean load(File f) {
		file = f;
		Loader loader = new Loader(this);
		boolean loaded = loader.loadFrom(f);
		if (!loaded) return false;
		changed = false;
		return true;
	}
    public String fileName() {
        String name = (changed) ? "*" : "";
        if (file == null) name += "Sans nom";
        else name += file.getName();
        return name; 
    }
	public void sort() {
		content.sort((ToDo t1, ToDo t2) -> t1.compareTo(t2));
	}
	public void performs(Operation op) throws Exception {
		op.doItOn(this, 0);
	}
	public void performs(Operation op, int margin) throws Exception {
		op.doItOn(this, margin);
	}
	public Iterator<ToDo> iterator() {
		return content.iterator();
	}
	public int size() {
		return content.size();
	}
	public void add(ToDo t) {
		content.add(t);
	}
	public void clear() {
		content.clear();
	}
	public void remove(ToDo t) {
		content.remove(t);
	}
	public <T> T[] toArray(T[] a) {
		return content.toArray(a);
	}
	public boolean isDone() {
		for(ToDo t : content) {
			if (!t.isDone())
				return false; 
		}
		return true;
	}
}
