package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import controler.ToDoListListener;
import controler.Controler;
import controler.DateControler;
import model.*;

@SuppressWarnings("serial")
public class ToDoListFrame extends JFrame {
    private static final String Title_End = " - Pense-bête";
    private ToDoList toDo;
    private JList<SimpleToDo> lstToDo;
    private JTextField txtName;
    private JTextField txtDate;
    private JCheckBox cbDone;
    private JButton deleteButton;
    private JButton modifyButton;
    private MenuBar menus;
    private StatusBar info;
    public ToDoListFrame(ToDoList l, Controler controler) {
    	toDo = l;
        updateTitle();
        RemoveCommand deleteCommand = new RemoveCommand(this, controler);
        ChangeCommand modifyCommand = new ChangeCommand(this, controler);
        menus = new MenuBar(controler, deleteCommand, modifyCommand);
        setJMenuBar(menus);

        lstToDo = new JList<SimpleToDo>();
        ToDoListListener alListener = new ToDoListListener(this, controler);
        lstToDo.addListSelectionListener(alListener);
        JScrollPane listScrollPane = new JScrollPane(lstToDo);
        listScrollPane.setPreferredSize(new Dimension(250, 80));
        add(listScrollPane, BorderLayout.CENTER);

        JPanel editToDoPanel = new JPanel();
        editToDoPanel.setLayout(new BoxLayout(editToDoPanel, BoxLayout.Y_AXIS));

        Dimension dimForLabel = new Dimension(60, 20);
        Dimension dimForName = new Dimension(400, 20);
        Dimension dimForDate = new Dimension(250, 20);

        JPanel namePanel = new JPanel();
        namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
        namePanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        JLabel nameLabel = new JLabel("Nom :");
        nameLabel.setMaximumSize(dimForLabel);
        namePanel.add(nameLabel);
        namePanel.add(Box.createRigidArea(new Dimension(10, 0)));
        txtName = new JTextField();
        txtName.setMinimumSize(dimForName);
        txtName.setMaximumSize(dimForName);
        namePanel.add(txtName);
        editToDoPanel.add(namePanel);

        JPanel dateEnabledPanel = new JPanel();
        dateEnabledPanel.setLayout(new BoxLayout(dateEnabledPanel, BoxLayout.X_AXIS));
        dateEnabledPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
        JLabel dateLabel = new JLabel("Pour le :");
        dateLabel.setMaximumSize(dimForLabel);
        dateEnabledPanel.add(dateLabel);
        dateEnabledPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        txtDate = new JTextField();
        txtDate.setMinimumSize(dimForDate);
        txtDate.setMaximumSize(dimForDate);
        dateEnabledPanel.add(txtDate);
        txtDate.addKeyListener(new DateControler(txtDate));
        cbDone = new JCheckBox("Fait");
        cbDone.setMaximumSize(dimForLabel);
        dateEnabledPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        dateEnabledPanel.add(cbDone);
        editToDoPanel.add(dateEnabledPanel);

        JPanel cmdPanel = new JPanel();
        cmdPanel.setLayout(new BoxLayout(cmdPanel, BoxLayout.LINE_AXIS));
        cmdPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        cmdPanel.add(Box.createHorizontalGlue());
        modifyButton = new JButton("Modifier");
        modifyButton.addActionListener(modifyCommand);
        cmdPanel.add(modifyButton);
        cmdPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        deleteButton = new JButton("Suppr.");
        deleteButton.addActionListener(deleteCommand);
        cmdPanel.add(deleteButton);
        editToDoPanel.add(cmdPanel);
        
        add(editToDoPanel, BorderLayout.EAST);
        
        info = new StatusBar();
        add(info, BorderLayout.SOUTH);
        pack();
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(controler);
        setLocationRelativeTo(null);
        
        disableEditor();
    }
    public SimpleToDo getSelectedToDo() {
    	return lstToDo.getSelectedValue();
    }
    public String getEditedName() {
    	return txtName.getText();
    }
    public String getEditedDate() {
    	return txtDate.getText();
    }
    public boolean getEditedDone() {
    	return cbDone.isSelected();
    }
    public void allowSaveItem(boolean b) {
    	menus.allowSaveItem(b);
    }
    public void updateToDoList() {
        lstToDo.setListData(toDo.toArray(new SimpleToDo[0]));
        info.setToDoListSize(toDo.size());
    }
    public void updateTitle() {
        setTitle(toDo.fileName() + Title_End);
    }
    public void disableEditor() {
    	txtName.setText("");
    	txtName.setEnabled(false);
    	txtDate.setText("");
    	txtDate.setBackground(Color.WHITE);
    	txtDate.setEnabled(false);
    	cbDone.setSelected(false);
    	cbDone.setEnabled(false);
    	deleteButton.setEnabled(false);
    	modifyButton.setEnabled(false);
    	menus.allowModifyDeleteItems(false);
    }
    public void editorForSelection() {
    	SimpleToDo t = lstToDo.getSelectedValue();
        if (t != null) {
	    	txtName.setText(t.name());
	    	txtName.setEnabled(true);
	    	if (t.dueDate() != null) txtDate.setText((t.dueDate()).toString());
	    	txtDate.setEnabled(true);
	    	cbDone.setEnabled(true);
	    	cbDone.setSelected(t.isDone());
	    	deleteButton.setEnabled(true);
	    	modifyButton.setEnabled(true);
	    	menus.allowModifyDeleteItems(true);
        }
    }
}
